import * as server_main from '@minecraft/server'
import * as uicontroller from './controllers/ui_controller.js';
import * as recruitment_controller from './controllers/recruitment_controller.js';
import * as command_controller from './controllers/command_controller.js';
import * as entity_functions from './controllers/entity_functions.js';
import * as data_convert from './controllers/data_convert.js';
import * as commander_controller from './controllers/commander_controller.js';

//TAGGING FOR TICK CHECKS
server_main.world.events.tick.subscribe((event) => {
    for (const player of server_main.world.getPlayers()) {
        if (player.hasTag("select_faction")) {
            uicontroller.UI_SELECTSIDES(player)            
        }
        else if (player.hasTag("command_ui")) {
            command_controller.open_commandpost(player, entity_functions.get_nearest_entityTAG_OBJECTIVE(player, 25, "command_post", "player_host", parseInt(player.id.toString().slice(5))))            
        }
        else if (player.hasTag("update_post")) {
            command_controller.update_commandpost(player)
        }
        else if (player.hasTag("command_settitlebar")) {
            uicontroller.UI_UpdateCommandHud(player)
        }
    }
})

server_main.world.events.worldInitialize.subscribe((event) => {   
    //COMMANDER/ENTITY CHOSEN FACTION
    if (!server_main.world.scoreboard.getObjective('chosen_faction'))
    {
        server_main.world.scoreboard.addObjective('chosen_faction', 'Faction')
    }
    //CONTAINS THE PLAYERS COMMANDER LEVEL
    if (!server_main.world.scoreboard.getObjective('command_level'))
    {
        server_main.world.scoreboard.addObjective('command_level', 'Commander Level')
    }
    //CONTAINS THE COMMAND "XP" OF THE PLAYER OR THE POPULATION REQUIREMENT OF A RECRUITED ENTITY
    if (!server_main.world.scoreboard.getObjective('command_points'))
    {
        server_main.world.scoreboard.addObjective('command_points', 'Commander Points')
    }
    //CONTAINS THE POPULATION HANDLER OF THE PLAYER
    if (!server_main.world.scoreboard.getObjective('command_population'))
    {
        server_main.world.scoreboard.addObjective('command_population', 'Current Population')
    }
    //CONTAINS THE PLAYERS AMOUNT OF RECRUITED ENTITIES
    if (!server_main.world.scoreboard.getObjective('command_count'))
    {
        server_main.world.scoreboard.addObjective('command_count', 'Current Recruited')
    }
    //COMMAND POSTS ID FOR PLAYER
    if (!server_main.world.scoreboard.getObjective('command_post_id'))
    {
        server_main.world.scoreboard.addObjective('command_post_id', 'Command Post')
    }
    //FOR THE ENTITIES THAT HAVE A PLAYER HOST OR COMMAND POST
    if (!server_main.world.scoreboard.getObjective('player_host'))
    {
        server_main.world.scoreboard.addObjective('player_host', 'Host')
    }
})
server_main.world.events.playerJoin.subscribe((event) => {
    event.player.addTag("command_settitlebar")
})



server_main.world.events.blockPlace.subscribe((event) => {
    if (event.block.typeId == "lotr:mordor_commandpost"){
        let loc = event.block.x.toString() + ' ' + event.block.y.toString() + ' ' + event.block.z.toString()
        if (data_convert.faction_name(entity_functions.get_entity_objective_score(event.player, "chosen_faction")) != 'mordor'){
            event.player.tell("Place a command post of your faction...")
            event.player.runCommandAsync('setblock ' + loc + ' air')
            event.player.runCommandAsync('give @p lotr:command_post_mordor_item 1')   
        }
        else if (entity_functions.check_valid_score(event.player, "command_post_id")){
            event.player.tell("Delete your old command post before trying to place a new one...")
            event.player.runCommandAsync('setblock ' + loc + ' air')
            event.player.runCommandAsync('give @p lotr:command_post_mordor_item 1')           
        }
        else{
            command_controller.init_commandpost(event.player, event.block, 'mordor')   
        }    
    }
    if (event.block.typeId == "lotr:gondor_commandpost"){
        let loc = event.block.x.toString() + ' ' + event.block.y.toString() + ' ' + event.block.z.toString()
        if (data_convert.faction_name(entity_functions.get_entity_objective_score(event.player, "chosen_faction")) != 'gondor'){
            event.player.tell("Place a command post of your faction...")
            event.player.runCommandAsync('setblock ' + loc + ' air')
            event.player.runCommandAsync('give @p lotr:command_post_gondor_item 1')   
        }
        else if (entity_functions.check_valid_score(event.player, "command_post_id")){
            event.player.tell("Delete your old command post before trying to place a new one!")
            event.player.runCommandAsync('setblock ' + loc + ' air')
            event.player.runCommandAsync('give @p lotr:command_post_gondor_item 1')           
        }
        else{
            command_controller.init_commandpost(event.player, event.block, 'gondor')   
        }  
    }
})
server_main.world.events.blockBreak.subscribe((event) => {
    if (event.brokenBlockPermutation.type.id == "lotr:mordor_commandpost"){
        let loc = new server_main.Location(event.block.x, event.block.y, event.block.z)
        var entity = entity_functions.get_nearest_entityTYPE_OBJECTIVE_LOC(loc, 2, "lotr:command_post", "player_host", event.player.id.toString().slice(5)) //not getting entity

        if (entity.id.toString().slice(5) == entity_functions.get_entity_objective_score(event.player, "command_post_id").toString()){ 
            event.player.runCommandAsync('scoreboard players set @s command_post_id 0')
            event.player.tell('Your command post is now erased...')
        }
    }

    if (event.brokenBlockPermutation.type.id == "lotr:gondor_commandpost"){
        let loc = new server_main.Location(event.block.x, event.block.y, event.block.z)
        var entity = entity_functions.get_nearest_entityTYPE_OBJECTIVE_LOC(loc, 2, "lotr:command_post", "player_host", event.player.id.toString().slice(5)) //not getting entity

        if (entity.id.toString().slice(5) == entity_functions.get_entity_objective_score(event.player, "command_post_id").toString()){ 
            event.player.runCommandAsync('scoreboard players set @s command_post_id 0')
            event.player.tell('Your command post is now erased...')
        }
    }
   
})





server_main.world.events.entityCreate.subscribe((event) =>{
    if (!event.entity){return}
    

    //SPAWNING A TAMED ENTITY FROM RECRUITING EGG
    if (entity_functions.isEntityRecruited(event.entity) && event.entity.hasComponent("minecraft:is_tamed")){
        //IGNORE IF STATIC
        if (event.entity.typeId == "throwable_pretamed_entity" || event.entity.typeId == "throwable_pretamed_entity_static"){return}
        //IGNORE IF ALREADY ASSIGNED A PLAYER HOST     
        if (entity_functions.check_valid_score(event.entity, "player_host")){return}
        //Find nearest player with same faction to be commander
        let commander = entity_functions.get_nearest_entityTYPE_OBJECTIVE(event.entity, 8, 'minecraft:player', 'chosen_faction', entity_functions.get_entity_objective_score(event.entity, 'chosen_faction'))
        if (!commander){ 
            server_main.world.say("No valid Commander found...")
            recruitment_controller.fail_spawn(event.entity) 
            return 
        }
        //Check if population limit has been reached
        var populationCost = entity_functions.get_entity_objective_score(event.entity, "command_population")
        if (recruitment_controller.recruit_check(commander, event.entity, populationCost)){
            event.entity.runCommandAsync('scoreboard players set @s player_host ' + commander.id.toString().slice(5))
            commander.runCommandAsync('scoreboard players add @s command_population ' + populationCost)
            commander.runCommandAsync('scoreboard players add @s command_count 1')
        }
        else
        {
            recruitment_controller.fail_spawn_c(event.entity, commander)
            return 
        }
    }
})
server_main.world.events.entityHurt.subscribe((event) =>{
    //On a death
    if (event.hurtEntity.getComponent("minecraft:health").current <= 0){

        //Cant lock in an attacker or dying entity
        if (!event.damagingEntity || !event.hurtEntity){return}

        //CALLED IF KILLED ENTITY HAS A SCORE FOR XP TO GAIN (HAS COMMAND POINTS SCORE + CHECK FACTION) - Faction 88 = WILD
        //Give XP to proper killer IF warented by faction (ADD IF TAMED ENTITY KILLS, GET TAG[1] PID AND ADD XP TO THAT PLAYER)
        if (entity_functions.check_valid_score(event.hurtEntity, "command_points")){
            var gained_xp = 0
            //If killing a player, get a portion of their XP
            if (event.hurtEntity.typeId == "minecraft:player"){
                gained_xp = entity_functions.get_entity_objective_score(event.hurtEntity, "command_level")
                gained_xp *= 8.2
            }
            else{
                gained_xp = entity_functions.get_entity_objective_score(event.hurtEntity, "command_points")
            }
            var gain_side = data_convert.FactionSIDEINVERT(entity_functions.get_entity_objective_score(event.hurtEntity, "chosen_faction"))
            var damager_side = data_convert.FactionSIDE(entity_functions.get_entity_objective_score(event.damagingEntity, "chosen_faction"))
            
            if (damager_side == gain_side || entity_functions.get_entity_objective_score(event.hurtEntity, "chosen_faction") == 88){
                //server_main.world.say("giving XP")
                //DIRECT TO PLAYER
                if (event.damagingEntity.typeId == "minecraft:player"){//give player xp for killing entity
                    commander_controller.modify_commanderXP(event.damagingEntity, gained_xp)
                }
                //A TAMED ENTITY KILLS IT
                else if (entity_functions.check_valid_score(event.damagingEntity, "player_host")){
                    gained_xp *= 0.7
                    var playerHost = entity_functions.get_playerBy_ID(entity_functions.get_entity_objective_score(event.damagingEntity, "player_host"))            
                    commander_controller.modify_commanderXP(playerHost, Math.round(Number(gained_xp)))
                }
            }
        }

        //Update the population of the Commander of the tamed entity IF entity is tamed player host
        if (entity_functions.check_valid_score(event.hurtEntity, "player_host")){
            var gained_pop = entity_functions.get_entity_objective_score(event.hurtEntity, "command_population")
            event.damagingEntity.runCommandAsync(('scoreboard players remove ' + entity_functions.get_playerBy_ID(entity_functions.get_entity_objective_score(event.hurtEntity, "player_host")).nameTag + ' command_population ' + gained_pop))
            event.damagingEntity.runCommandAsync(('scoreboard players remove ' + entity_functions.get_playerBy_ID(entity_functions.get_entity_objective_score(event.hurtEntity, "player_host")).nameTag + ' command_count 1'))          
        }



        //If a command post is destroyed, update the players command_post_id score
        if (event.hurtEntity.typeId == "lotr:command_post"){
            //player.runCommandAsync('scoreboard players set @e[type=lotr:command_post, c=1] player_host ' + (player.id.toString().slice(5)))
            var player = entity_functions.get_playerBy_ID(entity_functions.get_entity_objective_score(event.hurtEntity, "player_host"))
            world.say(player.nameTag.toString())
            player.runCommandAsync('scoreboard players set @s command_post_id 0')
        }
    }
    
})